import socket
# Socket API is used to send messages across network via TCP.

global s, hostIP, port
s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
# Creates socket object with arguments specifying address family & socket type.
# AF_INET is the Internet address family for IPv4.
# SOCK_STREAM is the socket type for TCP.
hostIP, port = '10.0.0.15', 1234
# Default HostIP and port for sending messages from client to server.


def client(message):
    """
    Allows a message to be sent from client to server via TCP,
    provided that the server is expecting a message,
    and that the bitlength of the message is less than or equal
    to that of which the server is expecting.

    Parameters:
    message(str): Message to be sent
    """
    s.connect((hostIP, port))
    # Establish connection to server and initiate three-way handshake.
    print(message)
    # Prints message to be sent to console
    s.send(message.encode())
    # Encodes message and sends it to the server.
    s.close
    # Socket closed to establish the completion of the message being sent.


def server_setup(listenDuration):
    """
    Establishes the server by setting up a listening socket
    on the same TCP port as that which the client uses to send messages.

    Parameters:
    listenDuration(int): Number of allowed unaccepted connections.
    """
    s.bind(('', port))
    # Associates the socket with a specific network interface and port.
    s.listen(listenDuration)
    # Enables the server to accept connections.


def server_recieve(bitLength):
    """
    Allows server to recieve message from client via TCP.

    Parameters:
    bitLength(int): Bit length of expected message to be received

    Returns:
    data(str): Recieved message
    """
    c, addr = s.accept()
    # Establish connection with client
    data = c.recv(bitLength).decode()
    # Accepts portion message, in accordance to the bitLength, and decodes it.
    c.close()
    # Socket closed to establish the completion of the message being recieved.
    return data


def setHostIP(IP):
    """
    Sets the Server IP address to desired value IP.

    Parameters:
    IP(str): An IP adress
    """
    global hostIP
    hostIP = IP
    # Replaces old IP address value with new IP argument.


def setPort(P):
    """
    Sets the TCP port to desired value P.
    Superuser privileges may be required for ports < 1024.
    If you are unsure, preferably use ports > 1023

    Parameters:
    P(int): A port number
    """
    global port
    port = P
    # Replaces old port value with new P argument.


def getHostIP():
    """
    Retreives the current Server IP address.

    Returns:
    hostIP(str): Current host IP
    """
    return hostIP


def getPort():
    """
    Retreives the current TCP port.

    Returns:
    hostIP(str): Current TCP port
    """
    return port



import RPi.GPIO as GPIO
# Allows for the implementation of the onboard RPi GPIO pins.
import time
# Used to generate delay.
import TCP
# Allows for the transmission of relevant messages via TCP.
import tempSensor

global buzzerPin
buzzerPin = 4
# Default GPIO pin for buzzer


def setup(motion):
    """
    Configures motion sensor for input in accordance to the GPIO pin provided.

    Parameters:
    motion(int): Motion sensor pin
    """
    GPIO.setmode(GPIO.BCM)
    # Sets pin numbering system
    GPIO.setup(motion, GPIO.IN)
    # Configures given pin for input usage.


def motion_detect(motion):
    """
    Determines if an individual triggers a motion sensor.

    Parameters:
    motion(int): Second motion sensor pin
    """
    if (GPIO.input(motion) == 1):
        print "Motion detected"
    else:
        print "No motion detected"


def direc_detect(motion_out, motion_in):
    """
    Determines the direction in which an individual is travelling,
    by assessing the sequence of triggers between two motion sensors.

    Parameters:
    motion_out(int): Outside motion sensor pin
    motion_in(int): Inside motion sensor pin
    """
    if (GPIO.input(motion_out) == 1):
        # Determines if outside motion sensor is triggered first.
        init_trig(motion_in)
        # Wait atleast 4 seconds for the corresponding sensor to trigger.
        ensure_low(motion_out, motion_in)
        # Ensure that both motion sensors are low/are not triggered.

    if (GPIO.input(motion_in) == 1):
        # Determines if inside motion sensor is triggered first.
        init_trig(motion_out)
        # Wait atleast 4 seconds for the corresponding sensor to trigger.
        ensure_low(motion_out, motion_in)
        # Ensure that both motion sensors are low/are not triggered.

def direc_detect_with_BuzzandTemp(motion_out, motion_in):
    """
    Determines the direction in which an individual is travelling,
    by assessing the sequence of triggers between two motion sensors.
    Includes buzzer and temperature sensor operation. 

    Parameters:
    motion_out(int): Outside motion sensor pin
    motion_in(int): Inside motion sensor pin
    """
    if (GPIO.input(motion_out) == 1):
        # Determines if outside motion sensor is triggered first.
        init_trig_with_BuzzandTemp(motion_in)
        # Wait atleast 4 seconds for the corresponding sensor to trigger.
        ensure_low(motion_out, motion_in)
        # Ensure that both motion sensors are low/are not triggered.

    if (GPIO.input(motion_in) == 1):
        # Determines if inside motion sensor is triggered first.
        init_trig(motion_out)
        # Wait atleast 4 seconds for the corresponding sensor to trigger.
        ensure_low_with_BuzzandTemp(motion_out, motion_in)
        # Ensure that both motion sensors are low/are not triggered.
        
def speed_detect(motion_first, motion_second, distance):
    """
    Determines the direction in which an individual is travelling,
    by assessing the time between the triggering of two motion sensors.

    Parameters:
    motion_out(int): Outside motion sensor pin
    motion_in(int): Inside motion sensor pin
    """
    if (GPIO.input(motion_first) == 1):
        # Determines if first motion sensor is triggered first.
        speed(timing(motion_second), distance)
        # Sends speed as message to server when corresponding sensor triggered.
        ensure_low(motion_first, motion_second)
        # Ensure that both motion sensors are low/are not triggered.

    if (GPIO.input(motion_second) == 1):
        # Determines if second motion sensor is triggered first.
        speed(timing(motion_first), distance)
        # Sends speed as message to server when corresponding sensor triggered.
        ensure_low(motion_first, motion_second)
        # Ensure that both motion sensors are low/are not triggered.


def init_trig_with_BuzzandTemp(motion):
    """
    Waits up to 4 seconds until corresponding sensor is triggered.
    If sensor triggered and temperature sensor not been triggered prior,
    buzzer will go off.
    If had triggered prior, direction of movement sent as message to server.

    Parameters:
    motion(int): Second motion sensor pin
    """
    for x in range(40):
        # Loop for 40 iterations.
        time.sleep(0.1)
        # Delay for 10ms.
        if (GPIO.input(motion) == 1):
            # Determines if corresponding motion sensor is triggered.
            if (motion == 1) and (!tempSensor.scanned):
                # Determines if user enters without temp scan.
                GPIO.setup(buzzerPin, GPIO.OUT)
                # Set buzzerPin to output mode.
                GPIO.output(buzzerPin, GPIO.HIGH)
                # Set buzzerPin high.
                time.sleep(2)
                # Delay for 2 seconds.
                GPIO.output(buzzerPin, GPIO.LOW)
                # Set buzzerPin low.
                print("BUZZ")
                break
            else:
                TCP.client(str(motion))
                # Sends direction of motion as message to the connected server.
                tempSensor.scanned = False
                break

def init_trig(motion):
    """
    Waits up to 4 seconds until corresponding sensor is triggered.
    motion(int): Second motion sensor pin
    """
    for x in range(40):
        # Loop for 40 iterations.
        time.sleep(0.1)
        # Delay for 10ms.
        if (GPIO.input(motion) == 1):
            # Determines if corresponding motion sensor is triggered.
                TCP.client(str(motion))
                # Sends direction of motion as message to the connected server.
                break

def timing(motion):
    """
    Returns calculated time between triggering of corresponding motion sensor
    after the first motion sensor is triggered. Accurate to 10ms.

    Parameters:
    motion(int): Second motion sensor pin

    Returns:
    tic - toc (float): Length of time in seconds
    """
    tic = time.perf_counter()
    # Establishes time in which first motion sensor is triggered.
    while (GPIO.input(motion) != 1):
        # Determines if corresponding motion sensor is triggered.
        time.sleep(0.01)
        # Delay by 10ms.

    toc = time.perf_counter()
    # Establishes time in which second motion sensor is triggered.
    return tic - toc


def speed(time, distance):
    """
    Calculates and sends speed as a message to server.

    Parameters:
    time(float): Time given in seconds
    distance(float): Distance given in meters
    """
    speed = distance/time
    # Calculates speed in meters/second.
    TCP.client(str(speed))
    # Sends speed as a message to the connected server.


def ensure_low(motion_out, motion_in):
    """
    Returns only when both motion sensors are low/are not triggered.

    Parameters:
    motion_out(int): Outside motion sensor pin
    motion_in(int): Inside motion sensor pin
    """
    while (GPIO.input(motion_out) == 1) or (GPIO.input(motion_in) == 1):
        # Determines if either of the two motion sensors are high/triggered.
        time.sleep(0.1)
        # Delay by 100ms.


def setBuzzerPin(Pin):
    """
    Sets the Buzzer GPIO Pin to desired Pin value

    Parameters:
    Pin(int): A pin number
    """
    global buzzerPin
    buzzerPin = Pin
    # Replaces old pin value with the new Pin argument.



import pigpio
import smbus2
import time
import crcmod.predefined
import TCP

global scanned, init
scanned = False
init = False


def setup(channel, omAddress, arraysize):
    """
    Enables I2C communication between Raspberry Pi and the temperature sensor.

    Parameters:
    channel(int): Raspberry Pi i2c channel
    omAddress(int): Omron I2C address
    arraysize(int): Size of thermal array
    """
    global BUFFER_LENGTH
    global tdata_raw
    global handle
    global tdata
    global pi
    BUFFER_LENGTH = (arraysize*2) + 3
    # Establish data buffer size.
    pi = pigpio.pi()
    bus = smbus2.SMBus(channel)
    # Create a bus for I2C communication.
    time.sleep(0.1)
    # Delay for 10ms.
    handle = pi.i2c_open(channel, omAddress)
    tdata_raw = [0]*BUFFER_LENGTH
    tdata = [0.0]*arraysize


def feverScanner(maxtemp):
    """
    Determines if an individual is standing infront of the temperature sensor,
    waits a full second and reads the temperature value again to gain a more
    stable value. Also determines if temperature is above that of the norm.
    This data is relayed as a message to the server.
    """
    global init
    global scanned
    tempReading()
    if (tdata[0] > roomtemp) and (tdata[0] > 30):
        if (!init):
            init = True
            feverScanner()
        else:
            init = False
            scanned = True
            TCP.client(str(tdata[0]))
            if (tdata[0]) > maxtemp:
                scanned = False
                time.sleep(1)


def tempReading():
    """
    Determines the temperature of an object in front of the temperature sensor.
    """
    global tdata
    time.sleep(1)
    (bytes_read, tdata_raw) = pi.i2c_read_device(handle, BUFFER_LENGTH)
    # Reads raw data and number of bytes read
    a = 0
    b = len(tdata_raw)-2
    for i in range(2, b, 2):
        # Converts raw data to detected individual temperature in degrees celsius.
        # Starts at second byte as first byte is used for room temperature.
        tdata[a] = float((tdata_raw[i+1] << 8) | tdata_raw[i])/10
        a += 1


def roomtemp():
    """
    Determines the room temperature.
    """
    global roomtemp
    tPAT = float((tdata_raw[1] << 8) | tdata_raw[0])/10
    # Converts raw data to room temperature in degrees celsius.
    # First byte of raw data represents room temperature.
    roomtemp = tPAT


def getroomtempValue():
    """
    Retreives the most recently determined room temperature reading.

    Returns:
    roomtemp(float): Room temperature in degrees celsius
    """
    return(roomtemp)


def gettempReadingValue():
    """
    Retreives the most recently determined temperature reading.

    Returns:
    tdata[0](float): Temperature in degrees celsius
    """
    return(tdata[0])
